<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/* * **************************************************
 * 	@author		Mian Saleem                 *
 * 	@package 	Simple Forum v3.0           *
 * 	@subpackage 	topic model                 *
 * ************************************************** */

class Topics_m extends CI_Model {

	public function __construct()
	{
		parent::__construct();
	}

	public function get_topics($limit, $offset=NULL)
	{
		$data = array(); 	

       	$this->db->select('topics.TopicID, topics.slug, topics.description, topics.category_slug, topics.TopicName, topics.CreatedBy, topics.LastPost, topics.CategoryID, topics.CreatedTime, topics.LastPostTime, topics.Sticky, topics.Closed, topics.Public, topics.Flagged, users.id, users.username, users.email')->from('topics');
       	$this->db->join('users', 'users.username = topics.CreatedBy'); // Joins users
	   	$this->db->order_by('topics.Sticky desc, topics.LastPostTime '.$this->session->userdata('topicsOrder').', topics.Closed');
	   	$this->db->limit($limit, $offset); // For pagination
		
		$agroup = array('admin', 'moderators');
		if($this->ion_auth->is_group($agroup)) {
			$this->db->where('topics.Active',1);
		} elseif($this->ion_auth->is_group('members')) {
			$this->db->where('topics.Public', 1)->or_where('topics.CreatedBy', $this->session->userdata('username'));
		} else {
			$this->db->where('topics.Active',1)->where('topics.Public', 1);
		}
		
       	//$options = array('topics.Active'=>'1', 'topics.CreatedBy'=>$this->session->userdata('username')); // Only get active topics
		//$this->db->select('*')->from('topcis')->where('Activie = 1 AND ', NULL, FALSE)
	   	//$q = $this->db->get_where('topics', $options); 
		$q = $this->db->get();
		if($q->num_rows() >0)
		{
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}

			return $data;
		}
		else
		{
			return false;
			// There must have being a problem, create a error log
			log_message('error', 'get_topics function failed! - /modules/topics/models/topics_m/get_topics');
		}
	}
	
	public function get_items()
    {
        $q = $this->db->get('items');
        
        if($q->num_rows() >0)
        {
            foreach ($q->result_array() as $row)
            {
                $data[] = $row;
            }
            
            return $data;
        } else {
            return false;
        }
    }
    
    public function get_topic($topic_id)
    {
        $data = array();
        $this->db->select('topics.TopicID, topics.slug, topics.category_slug, topics.description, topics.TopicName, topics.CreatedBy, topics.LastPost, topics.CategoryID, topics.CreatedTime, topics.LastPostTime, topics.Sticky, topics.Closed, topics.Public, topics.Flagged, comments.CommentID, comments.Body');
		$this->db->from('topics');
        $this->db->join('comments', 'comments.Title = topics.TopicName');
		$this->db->where('topics.TopicID', $topic_id);
		
		if(!$this->ion_auth->logged_in()) {
				$this->session->set_flashdata('error', $this->lang->line('errorPP'));				
					redirect('forums', 'refresh');
		}
		
		if(!$this->ion_auth->is_group(array('admin', 'moderators'))) {
			$this->db->where('topics.CreatedBy', $this->session->userdata('username'));
		} 
		
        $this->db->limit('1');
                
        $q = $this->db->get();
        
        if($q->num_rows() >0)
        {
            foreach ($q->result_array() as $row)
            {
                $data[] = $row;
            }
            
            return $data;
        } else {
            return false;
        }
    }

	public function count_topics()
	{
		$agroup = array('admin', 'moderators');
		if($this->ion_auth->is_group($agroup)) {
			$this->db->where('topics.Active',1);
		} elseif($this->ion_auth->is_group('members')) {
			$this->db->where('topics.public', 1)->or_where('topics.CreatedBy', $this->session->userdata('username'));
		} else {
			$this->db->where('topics.Active',1)->where('topics.public', 1);
		}
		
		$q = $this->db->get('topics');

		if($q->num_rows() >0)
		{
			return $q->num_rows();
		}
		else
		{
			return false;
		}
	}
	

	/**
    * Return a count of topics in a category
	*
	* @access public
	* @return $num_rows
	*/
    		
	public function count_cat_topics($category_id)
	{
		/*$agroup = array('admin', 'moderators');
		if($this->ion_auth->is_group($agroup)) {
			$this->db->where('topics.Active',1);
		} elseif($this->ion_auth->is_group('members')) {
			$this->db->where('topics.public', 1)->or_where('topics.CreatedBy', $this->session->userdata('username'));
		} else {
			$this->db->where('topics.Active',1)->where('topics.public', 1);
		}*/
		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		$this->db->where('topics.CategoryID',$category_id);
		
		$query = $this->db->get('topics');

		return $query->num_rows();	
	}	
	
	public function countCatTopicsByCatSlug($slug)
	{

		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		$this->db->where('topics.category_slug',$slug);
		
		$query = $this->db->get('topics');

		return $query->num_rows();	
	}
	
	public function topicCreatedBy($topic_id = NULL)
	{

   		$q = $this->db->get_where('topics', array('TopicID' => $topic_id), 1);
		if($q->num_rows() > 0)
		{
			$data =  $q->row() ;
			return $data->CreatedBy;
			
		}
		return false;
	}
	
	public function isTopicPublic($topic_id = NULL)
	{

   		$q = $this->db->get_where('topics', array('TopicID' => $topic_id, 'Public' => '1'), 1);
		if($q->num_rows() > 0)
		{
			return TRUE;
		} else {
			return FALSE;
		}
	}
	/**
    * Return all the topics in a category
	*
	* @access public
	* @param $category a string containing the category
	* @param $limit a string containing the limit
	* @param $offset a string containing the offset
	* @return $data
	*/		
	public function get_cat_topics($category_id, $limit, $offset=NULL)
	{
		$data = array(); 	

       	$this->db->select('topics.TopicID, topics.slug, topics.category_slug, topics.TopicName, topics.CreatedBy, topics.LastPost, topics.CategoryID, topics.CreatedTime, topics.LastPostTime, topics.Sticky, topics.Closed, topics.Flagged, users.id, users.username, users.email, topics.public');
   		$this->db->join('users', 'users.username = topics.CreatedBy');
   		$this->db->where('topics.CategoryID',$category_id)->where('topics.Active',1);
		
		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		
		$this->db->order_by('topics.Sticky desc, topics.LastPostTime '.$this->session->userdata('topicsOrder').'');
   		$this->db->limit($limit, $offset);

   		$q = $this->db->get('topics');

		if($q->num_rows() >0)
		{
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}
		}

		$q->free_result();

		return $data;
	}	
	
	public function getCatTopics($category, $limit, $offset=NULL)
	{
		$data = array(); 	

       	$this->db->select('topics.TopicID, topics.slug, topics.category_slug, topics.TopicName, topics.CreatedBy, topics.LastPost, topics.CategoryID, topics.CreatedTime, topics.LastPostTime, topics.Sticky, topics.Closed, topics.Flagged, users.id, users.username, users.email, topics.public');
   		$this->db->join('users', 'users.username = topics.CreatedBy');
   		$this->db->where('topics.Category_slug',$category)->where('topics.Active',1);
		
		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		
		$this->db->order_by('topics.Sticky desc, topics.LastPostTime '.$this->session->userdata('topicsOrder').'');
   		$this->db->limit($limit, $offset);

   		$q = $this->db->get('topics');

		if($q->num_rows() >0)
		{
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}
		}

		$q->free_result();

		return $data;
	}

    public function count_posts($topic_id)
    {
    	$options = array('TopicID'=>$topic_id, 'Active'=>'1');
    	$query = $this->db->get_where('comments', $options);
    	return $query->num_rows();
    }	

	public function submit_topic() 
	{
		// Store the page the user came from.

		$this->session->set_userdata('refered_from', $_SERVER['HTTP_REFERER']);
        
    	$this->form_validation->set_rules('title', 'Title', 'required|max_length[250]|xss_clean|trim');
    	$this->form_validation->set_rules('category', 'Category', 'required|is_natural_no_zero');
    	$this->form_validation->set_rules('comment', 'Comment', 'required|xss_clean|trim');

		if ($this->form_validation->run() == FALSE) 
		{
			$this->session->set_flashdata('error', validation_errors());
    		// Send the user back to the page they came from
			redirect($this->session->userdata('refered_from'));
		}
		else 
		{
		$TopicID = uniqid();	
		$TopicSlug = url_title($this->input->post('title'), '-', TRUE);
		if($this->getTopicBySlug($TopicSlug)) {
			$TopicSlug = $TopicSlug."-".$TopicID;
		}
		
		$data = array(			
			'username' => $this->session->userdata('username'),			
			'date' => time(),			
			'activity' => 'topic',			
			'topic_id' => $TopicID,		
			'topic_slug' => $TopicSlug,	
			'category_id' => $this->input->post('category'),            		
		);		

		$this->db->insert('activity', $data); // Insert activity into activity's table 

		if($this->input->post('Sticky') == '1')
		{
			$sticky = $this->input->post('Sticky');
		}
		else
		{
			$sticky = '0';
		}

		if($this->input->post('Close') == '1')
		{
			$close = $this->input->post('Close');
		}
		else
		{
			$close = '0';
		}
		
		if($this->input->post('Public') == '1')
		{
			$public = $this->input->post('Public');
		}
		else
		{
			$public = '0';
		}
		$category_slug = $this->getCatSlug($this->input->post('category'));
		
    	$data = array(
    		'TopicID' => $TopicID,
        	'TopicName'     =>  htmlspecialchars($this->input->post('title')),
        	'CreatedBy' => $this->session->userdata('username'),
        	'LastPost' => $this->session->userdata('username'),
        	'CategoryID'  =>  $this->input->post('category'),
			'category_slug'  =>  $category_slug,
        	'CreatedTime' => time(),
        	'LastPostTime' => time(),
            'Active' => '1',
			'Sticky'		=> $sticky,
			'Closed'			=> $close,
			'Public'			=> $public,
			'slug' => $TopicSlug,
			'description' => substr(preg_replace("/&#?[a-z0-9]{2,8};/i","",htmlspecialchars(strip_tags($this->input->post('comment')))), 0, 160),
    	);

    	$this->db->insert('topics', $data); 
		
    	$data = '';    	

    	$CommentID = uniqid();
		$commentBody = $this->ion_auth->clear_tags($this->input->post('comment'));
    	$data = array(
    		'CommentID' => $CommentID,
    		'TopicID' => $TopicID,
			'CategoryID' => $this->input->post('category'),
    		'Title' => htmlspecialchars($this->input->post('title')),
    		'Body' => $commentBody,
    		'CreatedBy' => $this->session->userdata('username'),
			'CategoryID'  =>  $this->input->post('category'),
    		'PostTime' => time(),
            'Active' => '1',
			'topic_slug' => $TopicSlug,	
			'category_slug'  =>  $category_slug,
   		);

   		$this->db->insert('comments', $data); 

		// Insert comment into comments table				
		//$this->ci =& get_instance();
		$this->load->config('ion_auth', TRUE);
		$message = $this->load->view('email_topic', $data, true);
		
		$config['mailtype'] = 'html';
        //$config['protocol'] = 'sendmail';
		//$config['mailpath'] = '/usr/sbin/sendmail';
		    
            $this->email->clear();
            $this->email->initialize($config);
            $this->email->to($this->config->item('admin_email', 'ion_auth'));
            $this->email->from($this->session->userdata('email'), ucfirst($this->session->userdata('username')));
            $this->email->subject('New discussion by '.ucfirst($this->session->userdata('username')));
            $this->email->message($message);
            $this->email->send(); 
			
			//echo $this->email->print_debugger(); die();
			
		$data = '';		

   		$this->session->set_flashdata('message', $this->lang->line('messageNewTopicSuccess'));

    	redirect('forums/posts/'.$category_slug.'/'.$TopicSlug.'/');
		}
	} // End of submitTopic
    
	/**
	 * Return the Topic's Name
	 *
	 * @access public
	 * @param $TopicID a string containing the topic's ID
	 * @return $row
	 */		
	public function getTopicBySlug($slug)
	{
		$q = $this->db->get_where('topics', array('slug' => $slug), 1);

		if($q->num_rows() > 0)
		{
			return $q->row();
		}
		return false;
	}
	
	public function get_topic_name($topic_id)
	{
		$this->db->select('TopicName');
		$options = array('TopicID'=>$topic_id);
		$q = $this->db->get_where('topics', $options);

		if($q->num_rows() > 0)
		{
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}
		}
		return $row['TopicName'];
	}
    
    public function get_topic_users($topic_id)
    {
        $emails = array();
        $this->db->select('users.email');
        $this->db->join('users', 'users.username = comments.CreatedBy');
        
        $options = array(
            'TopicID' => $topic_id,
        );
        
        $q = $this->db->get_where('comments', $options);
        
        if($q->num_rows() > 0)
        {
            foreach ($q->result_array() as $row)
            {
                $emails['emails'][] = $row['email'];
            }
        }
        return $emails;
    }
    
    public function get_cat_id($topic_id)
    {
        $this->db->select('CategoryID');
        $options = array(
            'TopicID' => $topic_id,
        );
        
        $q = $this->db->get_where('topics', $options);
        
        if($q->num_rows() > 0)
        {
            foreach ($q->result_array() as $row)
            {
                $data[] = $row;
            }
        }
        return $row['CategoryID'];
    }
	
	public function getCatSlug($category_id)
    {
        $this->db->select('slug');
        
        $q = $this->db->get_where('category', array('CategoryID' => $category_id));
        
        if($q->num_rows() > 0)
        {
            foreach ($q->result_array() as $row)
            {
                $data[] = $row;
            }
        }
        return $row['slug'];
    }
	
	public function getTopicSlug($topic_id)
    {
        
        $q = $this->db->get_where('topics', array('TopicID' => $topic_id), 1);
        
         if($q->num_rows() > 0)
        {         
            return $q->row();
        }
		return false;
    }


	 /**
	 * Return all the posts in a category 
	 *
	 * @access public
	 * @param $category a string contaning the category
	 * @param $limit a string containing the limit
	 * @param $offset a string containing the offset
	 * @param $TopicID a string containing the topic's ID
	 * @return $data
	 */	   
    public function get_posts($limit, $topic_id, $offset=NULL)
    {
		$data = array();
		$this->db->join('users', 'users.username = comments.CreatedBy');
		$this->db->join('topics', 'topics.TopicID = comments.TopicID');
		$this->db->order_by('comments.PostTime', 'asc'); 

		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		
		$this->db->limit($limit, $offset);   

		$options = array('comments.TopicID' => $topic_id, 'comments.Active'=>'1');

		$q = $this->db->get_where('comments', $options);

		if ($q->num_rows() > 0) 
		{
			foreach ($q->result_array() as $row) 
			{
				$data[] = $row;
			}
		}

		$q->free_result();
		return $data;
	}
	
	public function getPosts($limit, $topic_slug, $offset=NULL)
    {
		$data = array();
		$this->db->join('users', 'users.username = comments.CreatedBy');
		$this->db->join('topics', 'topics.slug = comments.topic_slug');
		$this->db->order_by('comments.PostTime', 'asc'); 

		$agroup = array('admin', 'moderators');
		if($this->ion_auth->logged_in() && !$this->ion_auth->is_group($agroup)) {
			$this->db->having('topics.public', 1)->or_having('topics.CreatedBy = "'.$this->session->userdata('username').'"', NULL);
		} elseif(!$this->ion_auth->logged_in()) {
			$this->db->having('topics.public', 1);
		}
		
		$this->db->limit($limit, $offset);   

		$options = array('comments.topic_slug' => $topic_slug, 'comments.Active'=>'1');

		$q = $this->db->get_where('comments', $options);

		if ($q->num_rows() > 0) 
		{
			foreach ($q->result_array() as $row) 
			{
				$data[] = $row;
			}
		}

		$q->free_result();
		return $data;
	}
    
    public function get_first_post($topic_id)
    {
        $data = array();
		if($this->isTopicPublic($topic_id))  {
		$this->db->select('*')->from('comments');
        $this->db->join('users', 'users.username = comments.CreatedBy');
        $this->db->join('topics', 'topics.TopicID = comments.TopicID');
		$this->db->where('comments.TopicID', $topic_id)->where('comments.Active', 1);	
			
		} else {
		$this->db->select('*')->from('comments');
        $this->db->join('users', 'users.username = comments.CreatedBy');
        $this->db->join('topics', 'topics.TopicID = comments.TopicID');
		$this->db->where('comments.TopicID', $topic_id)->where('comments.Active', 1);
		
			$agroup = array('admin', 'moderators');
			if($this->ion_auth->logged_in() && $this->ion_auth->is_group('members')) {
					$this->db->where('topics.CreatedBy', $this->session->userdata('username'));
			} elseif(! $this->ion_auth->logged_in()) {
				$this->db->where('topics.Public', 1);
			}
		}

		$this->db->order_by('comments.PostTime asc');
        $this->db->limit('1');
		
        $q = $this->db->get();
        
        if($q->num_rows() > 0)
        {
            foreach($q->result_array() as $row)
            {
                $data[] = $row;
            }
        }
        
        $q->free_result();
        return $data;
    }

	public function get_settings()
	{
		$this->db->select('*');

		$this->db->join('themes', 'themes.themeID = settings.themeID');

		$q = $this->db->get('settings', 1);

		if($q->num_rows() > 0)
		{	
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}
		}

		$q->free_result();

		return $data;
	}
    
    public function add_bookmark($topic_id, $topic_title)
    {
        // check and see if the user has already bookmarked this topic
        $user_id = $this->session->userdata('user_id');
        $check = $this->check_bookmark($topic_id, $user_id);

        if($check == false)
        {
            // The user already has this discussion bookmarked, let`s redirect them
            $this->session->set_flashdata('error', $this->lang->line('errorTopicBookmarked'));
            redirect($this->session->userdata('refered_from'));
        }
        else
        {
            $data = array(
                'bookmark_topic_title' => $topic_title,
                'bookmark_topic_id' => $topic_id,
                'bookmark_user_id' => $user_id,
            );
            
            $this->db->insert('bookmarks', $data);
            
            if($this->db->affected_rows() > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
    
    public function remove_bookmark($topic_id)
    {
        $user_id = $this->session->userdata('user_id');
        
        $data = array(
            'bookmark_topic_id' => $topic_id,
            'bookmark_user_id' => $user_id,
        );
        
        $this->db->delete('bookmarks', $data);
        
        if($this->db->affected_rows() > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    public function remove_discussion($topic_id)
    {
        if($this->ion_auth->is_admin() || $this->ion_auth->is_group('moderators'))
        {
            // If the admin is tryring to delete a topic then delete it.
            $data = array(
                'TopicID' => $topic_id,
            );
            
            $this->db->delete('topics', $data);
            $this->db->delete('comments', $data);
            
            if($this->db->affected_rows() > 0)
            {
                return true;
            }else{
                return false;
            }
        }else{
            // Perform checks
            $options = array(
                'TopicID' => $topic_id,
                'CreatedBy' => $this->session->userdata('username'),
            );
            
            $q = $this->db->get_where('topics', $options);
            
            if($q->num_rows() > 0)
            {
                $data = array(
                    'TopicID' => $topic_id,
                );
                
                $this->db->delete('comments', $data); 
                $this->db->delete('topics', $data);
                return true;
                
            }else{
                
                return false;
                
            }

            if($this->db->affected_rows() > 0)
            {
                
                return true;
                
            }else{
                
                return false;
                
            }
        }
    }
    
    public function check_bookmark($topic_id, $user_id)
    {
        // check and see if the user has already bookmarked this topic
        $this->db->select('*');
        $options = array(
            'bookmark_topic_id' => $topic_id,
            'bookmark_user_id' => $user_id,
        );
        
        $q = $this->db->get_where('bookmarks', $options);
        
        if($q->num_rows() > 0)
        {
            return false;
        }
        else
        {
            return true;
        }        
    }
	
	public function getEnvatoDetails() 
	{
		$q = $this->db->get('envato'); 
		  if( $q->num_rows() > 0 )
		  {
			return $q->row();
			
		  } 
		  return FALSE;
	}
	
	public function get_faqs()
	{
		$data = array(); 	
		if(!$this->ion_auth->logged_in()) {
			$this->db->where('visible', 1);
		}
		$q = $this->db->get('faqs');
		if($q->num_rows() >0)
		{
			foreach ($q->result_array() as $row)
			{
				$data[] = $row;
			}

			return $data;
		}
		else
		{
			return false;
			// There must have being a problem, create a error log
			log_message('error', 'get_faqs function failed! - /modules/topics/models/topics_m/get_faqs');
		}
	}
	
	public function getAllTopics() 
	{
		$q = $this->db->get('topics');
		if($q->num_rows() > 0) {
			foreach (($q->result()) as $row) {
				$data[] = $row;
			}
				
			return $data;
		}
	}
	
	public function getAllCategories() 
	{
		$q = $this->db->get('category');
		if($q->num_rows() > 0) {
			foreach (($q->result()) as $row) {
				$data[] = $row;
			}
				
			return $data;
		}
	}
	
	
}